/*
Created by: Nicholas Tolrud
Year: 2011
*/

import java.awt.geom.Point2D
import java.util.Vector

public class Animator implements Runnable
	private Vector<PointTransformation> pointTransformations
	private Thread th
	private VectorCanvas vectorCanvas
	
	public static final int ANIMATION_STEPS = 120
	public static final long SLEEP_TIME = 1000/60
	
	private SynchBoolean runningFlag
	
	public Animator(VectorCanvas vectorCanvas)
		this.vectorCanvas = vectorCanvas
		pointTransformations = new Vector<PointTransformation>()
		runningFlag = new SynchBoolean(false)
	
	public void addTransform(Point2D.Float pointToTransform, float destinationX, float destinationY)
		if !runningFlag.get()
			pointTransformations.add(new PointTransformation(pointToTransform, destinationX, destinationY))
	
	private void stepTransformations()
		for (PointTransformation transformation : pointTransformations)
			transformation.doStep()
		vectorCanvas.repaint()
	
	private void finishTransformations()
		for (PointTransformation transformation : pointTransformations)
			transformation.finish()
		vectorCanvas.repaint()
		
		pointTransformations.clear()
	
	private void sleep()
		try
			th.sleep(SLEEP_TIME)
		catch (InterruptedException e) {}
	
	public void run()
		if !runningFlag.get()
			runningFlag.set(true)
			th = Thread.currentThread()
			sleep()
		
			int numToRun = pointTransformations.size()
			for (int i = 1; i < ANIMATION_STEPS; i++)
				stepTransformations()
				sleep()
			finishTransformations()
			vectorCanvas.gui.enableButtons()
			runningFlag.set(false)