/*
Created by: Nicholas Tolrud
Year: 2011
*/

import java.util.Vector
import java.awt.geom.Point2D
import java.awt.Color

public class RedBlackTree
	private RedBlackTree_Node root
	private RedBlackTree_Node nil
	private int height
	
	//drawing
	private static float NODE_CIRCLE_SIZE = 10.0f
	private static float NODE_TEXT_SIZE = 10.0f
	private static float NODE_XSEPARATION = NODE_CIRCLE_SIZE + 2.0f
	private static float NODE_YSEPARATION = NODE_CIRCLE_SIZE + 20.0f
	
	private static Color NODE_TEXT_COLOR = Color.gray
	
	public RedBlackTree()
		this.nil = new RedBlackTree_Node(0)
		this.nil.p = nil
		this.nil.left = nil
		this.nil.right = nil
		this.nil.setColorBlack()
		this.root = nil
		this.height = 0
	
	public void recursive_updateHeight(RedBlackTree_Node node, int level)
		if node == this.nil
			if level > this.height
				this.height = level
			return
		
		recursive_updateHeight(node.left, level + 1)
		recursive_updateHeight(node.right, level + 1)
		
	public void updateHeight()
		if this.root == this.nil
			this.height = 0
			return
		
		height = 1
		recursive_updateHeight(this.root.left, 1)
		recursive_updateHeight(this.root.right, 1)
		
	
	public static RedBlackTree createFromKeyListString(String keyListString) throws NumberFormatException
		int[] keys = StringProcessing.parseIntArrayFromString(keyListString, ",")
		
		RedBlackTree tree = new RedBlackTree()
		for (int key : keys)
			tree.insertNode(new RedBlackTree_Node(key))
		
		tree.updateHeight()
		
		return tree
		
	
	public void leftRotate(RedBlackTree_Node x)
		RedBlackTree_Node y = x.right
		x.right = y.left
		if y.left != this.nil
			y.left.p = x
		y.p = x.p
		if x.p == this.nil
			this.root = y
		else if x == x.p.left
			x.p.left = y
		else
			x.p.right = y
		y.left = x
		x.p = y
	
	public void rightRotate(RedBlackTree_Node x)
		RedBlackTree_Node y = x.left
		x.left = y.right
		if y.right != this.nil
			y.right.p = x
		y.p = x.p
		if x.p == this.nil
			this.root = y
		else if x == x.p.right
			x.p.right = y
		else
			x.p.left = y
		y.right = x
		x.p = y
	
	public void insertFixUp(RedBlackTree_Node z)
		while z.p.isRed()
			if z.p == z.p.p.left
				RedBlackTree_Node y = z.p.p.right
				if y.isRed()
					z.p.setColorBlack()
					y.setColorBlack()
					z.p.p.setColorRed()
					z = z.p.p
				else 
					if z == z.p.right
						z = z.p
						leftRotate(z)
					z.p.setColorBlack()
					z.p.p.setColorRed()
					rightRotate(z.p.p)
			else
				RedBlackTree_Node y = z.p.p.left
				if y.isRed()
					z.p.setColorBlack()
					y.setColorBlack()
					z.p.p.setColorRed()
					z = z.p.p
				else 
					if z == z.p.left
						z = z.p
						rightRotate(z)
					z.p.setColorBlack()
					z.p.p.setColorRed()
					leftRotate(z.p.p)
				
		this.root.setColorBlack()
					
	
	public void insertNode(RedBlackTree_Node z)
		RedBlackTree_Node y = this.nil
		RedBlackTree_Node x = root
		while x != this.nil
			y = x
			if z.getKey() < x.getKey()
				x = x.left
			else
				x = x.right				
		z.p = y
		if y == this.nil
			this.root = z
		else if z.getKey() < y.getKey()
			y.left = z
		else
			y.right = z
		
		z.left = this.nil
		z.right = this.nil
		
		insertFixUp(z)
	
	public void recursiveDrawAndAnimate(RedBlackTree_Node node, Animator animator, VectorCanvas canvas, Vector<Vect_Object> vectObjects, int level, Point2D.Float currentPos, Point2D.Float parentPos)
		if node == this.nil
			return
			
		Color circleColor
		if node.isRed()
			circleColor = Color.red
		else
			circleColor = Color.black
		
		
		vectObjects.add(new Vect_Circle(currentPos, NODE_CIRCLE_SIZE, circleColor))
		vectObjects.add(new Vect_Text(currentPos, NODE_TEXT_COLOR, "" + node.getKey()))
		vectObjects.add(new Vect_NodeConnectorCurve(parentPos, currentPos))
		
		int level_PlusOne = level + 1
		
		Point2D.Float leftNodePos = new Point2D.Float(currentPos.x - ((float) Math.pow(2.0f, this.height - level_PlusOne))*NODE_XSEPARATION, currentPos.y + (this.height - level_PlusOne) * NODE_YSEPARATION)
		Point2D.Float rightNodePos = new Point2D.Float(currentPos.x + ((float) Math.pow(2.0f, this.height - level_PlusOne))*NODE_XSEPARATION, currentPos.y +  (this.height - level_PlusOne) * NODE_YSEPARATION)
		
		recursiveDrawAndAnimate(node.left, animator, canvas, vectObjects, level_PlusOne, leftNodePos, currentPos)
		recursiveDrawAndAnimate(node.right, animator, canvas, vectObjects, level_PlusOne, rightNodePos, currentPos)
		
		//adjust canvas boundaries
		if canvas.MIN_GLOBAL_X > currentPos.x
			canvas.MIN_GLOBAL_X = currentPos.x
		else if canvas.MAX_GLOBAL_X < currentPos.x
			canvas.MAX_GLOBAL_X = currentPos.x
		
		if canvas.MAX_GLOBAL_Y < currentPos.y
			canvas.MAX_GLOBAL_Y = currentPos.y
		
		//for animation
		float destinationX = currentPos.x
		float destinationY = currentPos.y
		currentPos.x = 0.0f
		currentPos.y = 0.0f
		
		animator.addTransform(currentPos, destinationX, destinationY)
		
	public Vector<Vect_Object> getDrawingObjectsAndAddAnimationToAnimator(Animator animator, VectorCanvas canvas)
		Vector<Vect_Object> vectObjects = new Vector<Vect_Object>()
		
		if root == this.nil
			return vectObjects
		
		Color circleColor
		if root.isRed()
			circleColor = Color.red
		else
			circleColor = Color.black
		
		Point2D.Float rootPos = new Point2D.Float(0.0f,0.0f)
		
		vectObjects.add(new Vect_Circle(rootPos, NODE_CIRCLE_SIZE, circleColor))
		vectObjects.add(new Vect_Text(rootPos, NODE_TEXT_COLOR, "" + root.getKey()))
		
		Point2D.Float leftNodePos = new Point2D.Float(-((float) Math.pow(2.0f, this.height - 1))*NODE_XSEPARATION, (this.height - 1) * NODE_YSEPARATION)
		Point2D.Float rightNodePos = new Point2D.Float(((float) Math.pow(2.0f, this.height - 1))*NODE_XSEPARATION, (this.height - 1) * NODE_YSEPARATION)
		
		canvas.MIN_GLOBAL_X = -500.0f
		canvas.MAX_GLOBAL_X = 500.0f
		canvas.MAX_GLOBAL_Y = 500.0f
		
		recursiveDrawAndAnimate(root.left, animator, canvas, vectObjects, 1, leftNodePos, rootPos)
		recursiveDrawAndAnimate(root.right, animator, canvas, vectObjects, 1, rightNodePos, rootPos)
		
		
		
		return vectObjects	