/*
Created by: Nicholas Tolrud
Year: 2011
*/

import javax.swing.JPanel
import javax.swing.JButton
import javax.swing.JTextField
import javax.swing.JLabel
import javax.swing.JSlider
import java.awt.Container

import java.awt.Component
import javax.swing.BoxLayout

import javax.swing.SwingConstants

import java.awt.event.ActionListener
import java.awt.event.ActionEvent
import javax.swing.event.ChangeListener
import javax.swing.event.ChangeEvent

import java.awt.Dimension

import java.util.Vector

public class RedBlackTreeGUI extends JPanel implements ActionListener,ChangeListener
	private static final long serialVersionUID = 78912347961L;
	private static final String ACTION_MAKE_TREE_FROM_TEXT = "0"
	private VectorCanvas canvas
	
	private JPanel controlsPanel
	private Vector<JButton> buttons
	private JTextField textField
	private JLabel statusLabel
	private JSlider zoomSlider

	public RedBlackTreeGUI()
		buttons = new Vector<JButton>()
		
		this.addComponents()
	
	private void addVectorCanvas()
		canvas = new VectorCanvas(this)
		canvas.setPreferredSize(new Dimension(600,600))
		this.add(canvas)
	
	
	private void addButton(String text, String ActionCommand)
		JButton button = new JButton(text)
		button.setActionCommand(ActionCommand)
		button.addActionListener(this)
		button.setAlignmentX(Component.CENTER_ALIGNMENT)
		controlsPanel.add(button)
		buttons.add(button)
	
	private void addTextField()
		textField = new JTextField()
		textField.setPreferredSize(new Dimension(200,40))
		textField.setAlignmentX(Component.CENTER_ALIGNMENT)
		controlsPanel.add(textField)
	
	private void addStatusLabel()
		statusLabel = new JLabel("waiting for input...")
		statusLabel.setAlignmentX(Component.CENTER_ALIGNMENT)
		statusLabel.setHorizontalTextPosition(SwingConstants.CENTER) 
		controlsPanel.add(statusLabel)
	
	public static float getZoomFromSliderValue(int value)
		if value == 0
			return VectorCanvas.MIN_ZOOM
		else if value == 100
			return VectorCanvas.MAX_ZOOM
			
		float zoom = (float) (Math.pow((value - VectorCanvas.MIN_ZOOM)/50.0, 1.0/VectorCanvas.ZOOM_SLIDER_POWER_FACTOR) + VectorCanvas.MIN_ZOOM)
		
		if zoom > VectorCanvas.MAX_ZOOM
			zoom = VectorCanvas.MAX_ZOOM
		else if zoom < VectorCanvas.MIN_ZOOM
			zoom = VectorCanvas.MIN_ZOOM
		
		return zoom
	
	public static int getSliderValueFromZoom(float zoom)
		if zoom == VectorCanvas.MIN_ZOOM
			return 0
		else if zoom == VectorCanvas.MAX_ZOOM
			return 100
			
		int value = (int) (50.0 * Math.pow(zoom - VectorCanvas.MIN_ZOOM, VectorCanvas.ZOOM_SLIDER_POWER_FACTOR) + VectorCanvas.MIN_ZOOM)
		
		if value < 0
			value = 0
		else if value > 100
			value = 100
		
		return value
	
	public void updateSlider(float zoom)
		zoomSlider.setValue(getSliderValueFromZoom(zoom))
	
	private void updateZoom(int sliderValue)
		canvas.setZoom(getZoomFromSliderValue(sliderValue))
		canvas.repaint()
	
	private void addZoomSlider()
		JLabel sliderLabel = new JLabel("Zoom")
		sliderLabel.setAlignmentX(Component.CENTER_ALIGNMENT)
		sliderLabel.setHorizontalTextPosition(SwingConstants.CENTER) 
		controlsPanel.add(sliderLabel)
	
		zoomSlider = new JSlider(0, 100) 
		zoomSlider.setAlignmentX(Component.CENTER_ALIGNMENT)
		zoomSlider.setValue(getSliderValueFromZoom(1.0f)) 
		zoomSlider.addChangeListener(this)
		controlsPanel.add(zoomSlider)
	
	private void addControlsPanel()
		controlsPanel = new JPanel()
		BoxLayout layout = new BoxLayout(controlsPanel, BoxLayout.Y_AXIS)
		controlsPanel.setLayout(layout)
		addZoomSlider()
		addStatusLabel()
		addTextField()
		addButton("Build tree", ACTION_MAKE_TREE_FROM_TEXT)
		this.add(controlsPanel)
	
	private void addComponents()
		addVectorCanvas()
		addControlsPanel()
	
	public synchronized void enableButtons()
		for (JButton button : buttons)
			button.setEnabled(true)
	
	public void disableButtons()
		for (JButton button : buttons)
			button.setEnabled(false)
	
	public void setStatusMessage(String message)
		statusLabel.setText(message)
	
	public void actionPerformed(ActionEvent e)
		if e.getActionCommand() == ACTION_MAKE_TREE_FROM_TEXT
			setStatusMessage("Built new tree")
			canvas.createTree(textField.getText())
	
	public void stateChanged(ChangeEvent e)
		updateZoom(zoomSlider.getValue())
	
	