/*
Created by: Nicholas Tolrud
Year: 2011
*/

import javax.swing.JPanel
import java.awt.Graphics
import java.awt.Dimension
import java.awt.Color

import java.util.LinkedList
import java.util.Vector

import java.awt.event.MouseWheelListener
import java.awt.event.MouseWheelEvent
import java.awt.event.MouseMotionListener
import java.awt.event.MouseEvent

import java.awt.geom.Point2D

public class VectorCanvas extends JPanel implements MouseWheelListener, MouseMotionListener
	private static final long serialVersionUID = 7861789123961L;
	
	private LinkedList<Vect_Object> vectorObjectList
	
	private float globalX
	private float globalY
	
	private float zoom
	
	
	public static final float MAX_ZOOM = 10.0f
	public static final float MIN_ZOOM = 0.2f
	public static final double ZOOM_SLIDER_POWER_FACTOR = Math.log(2.0)/Math.log(MAX_ZOOM - MIN_ZOOM)
	
	private static final float MIN_GLOBAL_Y = -100.0f
	
	//these used to be constants, but I wanted to change them depending on the size of the graph
	float MIN_GLOBAL_X = -500.0f
	float MAX_GLOBAL_X = 500.0f
	float MAX_GLOBAL_Y = 1500.0f
	
	private int lastMousePosX = 0
	private int lastMousePosY = 0
	private boolean dragging = false
	
	private Point2D.Float testPoint
	
	private boolean testBoolean
	private Animator animator
	
	public RedBlackTreeGUI gui
	
	public VectorCanvas(RedBlackTreeGUI gui)
		this.gui = gui
	
		this.setBackground(Color.white)
		vectorObjectList = new LinkedList<Vect_Object>()
		globalX = 0.0f
		globalY = 0.0f
		zoom = 1.0f
		addMouseWheelListener(this)
		addMouseMotionListener(this)
		
		animator = new Animator(this)
		testBoolean = true
		
		/*Point2D.Float rootPoint = new Point2D.Float(0.0f,0.0f)
		//testing
		testPoint = rootPoint
		
		Point2D.Float node2Point = new Point2D.Float(-120.0f,30.0f)
		Point2D.Float node3Point = new Point2D.Float(120.0f,30.0f)
		this.addVectorObject(new Vect_Circle(rootPoint,10.0f,Color.black))
		this.addVectorObject(new Vect_Text(rootPoint, Color.gray, "23"))
		
		this.addVectorObject(new Vect_Circle(node2Point,10.0f,Color.red))
		this.addVectorObject(new Vect_NodeConnectorCurve(rootPoint,node2Point))
		
		this.addVectorObject(new Vect_Circle(new Point2D.Float(120.0f,30.0f),10.0f,Color.red))
		this.addVectorObject(new Vect_NodeConnectorCurve(rootPoint,node3Point))
		*/
		
		/*this.addVectorObject(new Vect_Circle(-60.0f,60.0f,10.0f,Color.black))
		this.addVectorObject(new Vect_NodeConnectorCurve(-120.0f,30.0f,-60.0f,60.0f))
		
		this.addVectorObject(new Vect_Circle(-180.0f,60.0f,10.0f,Color.black))
		this.addVectorObject(new Vect_NodeConnectorCurve(-120.0f,30.0f,-180.0f,60.0f))
		
		this.addVectorObject(new Vect_Circle(60.0f,60.0f,10.0f,Color.black))
		this.addVectorObject(new Vect_NodeConnectorCurve(120.0f,30.0f,60.0f,60.0f))
		
		this.addVectorObject(new Vect_Circle(-30.0f,90.0f,10.0f,Color.red))
		this.addVectorObject(new Vect_NodeConnectorCurve(-60.0f,60.0f,-30.0f,90.0f))
		
		this.addVectorObject(new Vect_Circle(-90.0f,90.0f,10.0f,Color.red))
		this.addVectorObject(new Vect_NodeConnectorCurve(-60.0f,60.0f,-90.0f,90.0f))*/
	
	//utilities for drawing
	public static boolean isInCanvas(int canvasWidth, int canvasHeight, int minX, int minY, int maxX, int maxY)
		if minX > canvasWidth or maxX < 0 or minY > canvasHeight or maxY < 0
			return false
		else
			return true
	
	public static boolean isInCanvas(int canvasWidth, int canvasHeight, int x, int y)
		if x > canvasWidth or x < 0 or y > canvasHeight or y < 0
			return false
		else
			return true
	
	public static int getRelativePositionValue(float globalViewPositionValue, float zoom, float positionValue)
		return (int) ((positionValue  - globalViewPositionValue) * zoom) + 300
	
	public static void drawPoint(Graphics g, int x, int y)
		g.fillRect( x, y, 1, 1 )
	
	//misc
	public void setZoom(float zoomValue)
		this.zoom = zoomValue
	
	//component drawing
	public void paintComponent(Graphics g)
		Dimension canvasDimension = this.getSize()
		int canvasWidth = canvasDimension.width
		int canvasHeight = canvasDimension.height
		
		g.clearRect(0, 0, canvasWidth, canvasHeight) 
		
		for (Vect_Object o : vectorObjectList)
			o.draw(g, globalX, globalY, zoom, canvasWidth, canvasHeight)
		g.dispose()
	
	//mouse wheel listener event
	public void mouseWheelMoved(MouseWheelEvent e) 
		int wheelRotation = e.getWheelRotation()
		if wheelRotation < 0
			zoom *= 1.2f
			if zoom > MAX_ZOOM
				zoom = MAX_ZOOM
		else
			zoom *= 0.8f
			if zoom < MIN_ZOOM
				zoom = MIN_ZOOM
		
		gui.updateSlider(zoom)
		repaint()
	//mouse movement events
	public void mouseMoved(MouseEvent mme)
		dragging = false
	public void mouseDragged(MouseEvent mme)
		int mouseX = mme.getX()
		int mouseY = mme.getY()
		if dragging
			int diffX = mouseX - lastMousePosX
			int diffY = mouseY - lastMousePosY
			
			globalX -= diffX / zoom
			if globalX < MIN_GLOBAL_X
				globalX = MIN_GLOBAL_X
			else if globalX > MAX_GLOBAL_X
				globalX = MAX_GLOBAL_X
				
			globalY -= diffY / zoom
			if globalY < MIN_GLOBAL_Y
				globalY = MIN_GLOBAL_Y
			else if globalY > MAX_GLOBAL_Y
				globalY = MAX_GLOBAL_Y
			
			repaint()
			
		lastMousePosX = mouseX
		lastMousePosY = mouseY
		dragging = true
	
	
	//Vector Data Manipulation
	public void addVectorObject(Vect_Object o)
		int depth = o.depth
		int insertPos = 0
		for (Vect_Object obj : vectorObjectList)
			if obj.depth <= depth
				break
			insertPos++
		vectorObjectList.add(insertPos,o)
	
	public void removeVectorObject(Vect_Object o)
		vectorObjectList.remove(o)
	
	public void createTree(String input)
		vectorObjectList.clear()
		RedBlackTree tree
		try
			tree = RedBlackTree.createFromKeyListString(input)
		catch (NumberFormatException e)
			gui.setStatusMessage("Error: Given an invalid input.")
			return
		
		Vector<Vect_Object> newVectObjects = tree.getDrawingObjectsAndAddAnimationToAnimator(animator, this)
		
		for (Vect_Object o : newVectObjects)
			addVectorObject(o)
		
		globalX = 0.0f
		globalY = 0.0f
		
		//repaint()
		
		gui.disableButtons()
		Thread newThread = new Thread(animator)
		newThread.start()